/*
mob_eeprom.c

Copyright 2001 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#include "avrx.h"
#include "MotorTask.h"
#include "LcdTask.h"
#include "eeprom.h"
#include "mob_eeprom.h"

// Declare the programmed default modes
//
// If you maintain the "CheckSum" value, below, the system will
// boot after programming with the EEPROM valid.  Otherwize,
// make sure the values in "ResetEEPROM()" are the defaults you
// want when the EEPROM is initialized.
//
// These values will be set in the mob.eep file by the make
// process ready to be programmed into the chip.

unsigned char EEPROM
                eePWM_Mode   = SIGN_MAGNITUDE,
                eePWM_Rate   = PWM2K,
                eeLeftSupply = 0,
                eeRightSupply = 0,
                eeMixedMode = STRAIGHT,
                eeRC0low    = RC_MIN,  // Scaling factors for R/C
                eeRC0high   = RC_MAX,  // pulse range.  See MotorTask.c
                eeRC1low    = RC_MIN,
                eeRC1high   = RC_MAX,
                eeRC2low    = RC_MIN,
                eeRC2high   = RC_MAX,
                eeDeadBand  = 2,
				eeFlip		= 0,	// default is don't sense flip
                eefDisable  = (unsigned char) DEFAULT_DISABLE,     // Default is to kill
                eeBrakeMode = 0,
                eeSlewRate  = 0,
                eeChkSum    = (unsigned char) (
                              - SIGN_MAGNITUDE
                              - PWM2K
                              - 0
                              - 0
                              - STRAIGHT
                              - RC_MIN
                              - RC_MAX
                              - RC_MIN
                              - RC_MAX
                              - RC_MIN
                              - RC_MAX
                              - 2
							  - 0
                              - DEFAULT_DISABLE
                              - 0
                              - 0
                              );

unsigned char EEpromResetFlag;

// This routine resets the EEPROM when corrupt, to know good default values.
// This should match the above declarations so either a commanded eeprom 
// reset or a fresh program/eeprom load will result in the same values

void ResetEEPROM(void)
{
    PWM_Mode    = SIGN_MAGNITUDE;
    PWM_Rate    = PWM2K;
    LeftSupply  = 0;
    RightSupply = 0;
    MixedMode   = STRAIGHT;
    RC0.low     = RC_MIN;  // Scaling factors for R/C
    RC0.high    = RC_MAX;  // pulse range.  See MotorTask.c
    RC1.low     = RC_MIN;
    RC1.high    = RC_MAX;
    RC2.low     = RC_MIN;
    RC2.high    = RC_MAX;
    DeadBand    = 2;
	Flip_Mode	= 0;
    fDisable.Output = DEFAULT_DISABLE;     // Default is to kill
    SlewRate    = 0;
    BrakeMode   = 0;
    
    EEpromResetFlag = FALSE;
    WriteEEPROMInitModes();
}

// Initialize globals/validate modes.  No validation done yet.

void ReadEEPROMandInitModes(void)
{
    unsigned char CheckSum = 0;

    PWM_Mode    = eeprom_rb((unsigned)&eePWM_Mode);
    CheckSum    -= PWM_Mode;
    PWM_Rate    = eeprom_rb((unsigned)&eePWM_Rate);
    CheckSum    -= PWM_Rate;
    LeftSupply  = eeprom_rb((unsigned)&eeLeftSupply);
    CheckSum    -= LeftSupply;
    RightSupply = eeprom_rb((unsigned)&eeRightSupply);
    CheckSum    -= RightSupply;
    MixedMode   = eeprom_rb((unsigned)&eeMixedMode);
    CheckSum    -= MixedMode;
    RC0.low     = eeprom_rb((unsigned)&eeRC0low);
    CheckSum    -= RC0.low;
    RC0.high    = eeprom_rb((unsigned)&eeRC0high);
    CheckSum    -= RC0.high;
    RC1.low     = eeprom_rb((unsigned)&eeRC1low);
    CheckSum    -= RC1.low;
    RC1.high    = eeprom_rb((unsigned)&eeRC1high);
    CheckSum    -= RC1.high;
    RC2.low     = eeprom_rb((unsigned)&eeRC2low);
    CheckSum    -= RC2.low;
    RC2.high    = eeprom_rb((unsigned)&eeRC2high);
    CheckSum    -= RC2.high;
	DeadBand	= eeprom_rb((unsigned)&eeDeadBand);
	CheckSum	-= DeadBand;
	Flip_Mode	= eeprom_rb((unsigned)&eeFlip);
	CheckSum	-= Flip_Mode;
    fDisable.Output    = eeprom_rb((unsigned)&eefDisable);
    CheckSum    -= fDisable.Output;
    SlewRate    = eeprom_rb((unsigned)&eeSlewRate);
    CheckSum    -= SlewRate;
    BrakeMode   = eeprom_rb((unsigned)&eeBrakeMode);
    CheckSum    -= BrakeMode;

    CheckSum    -= eeprom_rb((unsigned)&eeChkSum);

    if (CheckSum != 0)
    {
        EEpromResetFlag = TRUE;
    }
}

void WriteEEPROMInitModes(void)
{
    unsigned char CheckSum = 0;
    
    eeprom_wb((unsigned)&eePWM_Mode, PWM_Mode);
    CheckSum -= PWM_Mode;
    eeprom_wb((unsigned)&eePWM_Rate,PWM_Rate);
    CheckSum -= PWM_Rate;
    eeprom_wb((unsigned)&eeLeftSupply,LeftSupply);
    CheckSum -= LeftSupply;
    eeprom_wb((unsigned)&eeRightSupply, RightSupply);
    CheckSum -= RightSupply;
    eeprom_wb((unsigned)&eeMixedMode, MixedMode);
    CheckSum -= MixedMode;
    eeprom_wb((unsigned)&eeRC0low, RC0.low);
    CheckSum -= RC0.low;
    eeprom_wb((unsigned)&eeRC0high, RC0.high);
    CheckSum -= RC0.high;
    eeprom_wb((unsigned)&eeRC1low, RC1.low);
    CheckSum -= RC1.low;
    eeprom_wb((unsigned)&eeRC1high, RC1.high);
    CheckSum -= RC1.high;
    eeprom_wb((unsigned)&eeRC2low, RC2.low);
    CheckSum -= RC2.low;
    eeprom_wb((unsigned)&eeRC2high, RC2.high);
    CheckSum -= RC2.high;
	eeprom_wb((unsigned)&eeDeadBand, DeadBand);
	CheckSum -= DeadBand;
	eeprom_wb((unsigned)&eeFlip, Flip_Mode);
	CheckSum -= Flip_Mode;
    eeprom_wb((unsigned)&eefDisable, fDisable.Output);
    CheckSum -= fDisable.Output;
	eeprom_wb((unsigned)&eeSlewRate, SlewRate);
	CheckSum -= SlewRate;
	eeprom_wb((unsigned)&eeBrakeMode, BrakeMode);
	CheckSum -= BrakeMode;

    eeprom_wb((unsigned)&eeChkSum, CheckSum);
}


